/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <malloc.h>
#include <gpib/ib.h>
#include "analyzer.h"
#include "my_stdio.h"
#include "ska_survey.h"
#include "observ_table.h"

#define MY_IBWRT(dev, fmt, arg...)\
        if ( my_ibwrt(dev, fmt, ## arg) & ERR ){ return -1; } 



static int my_ibwrite_generic(
	int (*write_func)(int ud, const void *data, long num_bytes),
	int dev, const char *fmt, ...)
{
	va_list ap;
	int l = -1;
	int n, size = 80;
	char *buff = NULL;

	if ((buff = malloc (size)) == NULL) return l;
	while (1) {
		/* Try to print in the allocated space. */
		va_start(ap, fmt);
		n = vsnprintf (buff, size, fmt, ap);
		va_end(ap);
		if (n > -1 && n < size) {
			break;
		}
		else {
			/* Else try again with more space. */
			if (n > -1)    /* glibc 2.1 */
				size = n+1; /* precisely what is needed */
			else           /* glibc 2.0 */
				size *= 2;  /* twice the old size */
			if ((buff = realloc (buff, size)) == NULL)
				return l;
		}
	}

	if (buff != NULL) {
		PVERB(PINFO,"# Send: %s\n", buff);
		l = write_func(dev, buff, n);
#ifdef VERBOSE_MODE
		if (l & ERR) {
			PVERB(PERR,"Error sending: %s [ibstat: 0x%x]\n", 
			      buff, l);
		}
#endif
		free(buff);
	}

	return l;
}


int my_ibwrt(int dev, const char *fmt, ...)
{
	return my_ibwrite_generic(ibwrt, dev, fmt);
}


int my_ibwrta(int dev, const char *fmt, ...)
{
	return my_ibwrite_generic(ibwrta, dev, fmt);
}



int open_analyzer(int pad)
{
	int dev;
	short listen;

	dev = ibdev(AN_GPIB, pad, 0, AN_TIME_OUT, 1, 0);
	/* check if listener is present */
	ibln(AN_GPIB, pad, NO_SAD, &listen );
	if(ibsta & ERR){
		if(iberr == EBUS){
			PVERB(PERR,
			      "Analyzer not found in primary address %d\n",
			      pad);
			SYSLOG(LOG_ERR,
			       "Analyzer not found in primary address %d",
			       pad);
			return -1;
		}
		SYSLOG(LOG_ERR,"Analyzer: init error [0x%x]", iberr);
		return -1;
	}
	if( ! listen ){
		PVERB(PERR,
		      "Analyzer not found in primary address %d\n",
		      pad);
		SYSLOG(LOG_ERR,
		       "Analyzer not found in primary address %d",
		       pad);
		return -1;
	}
	/* IP: Instrument Preset */
	MY_IBWRT(dev, "IP;");
	/* SNGLS: Single Sweep */
	MY_IBWRT( dev, "SNGLS;TS;");

	/* Automatic serial polling is enabled. */
	ibconfig(AN_GPIB, IbcAUTOPOLL, 1);

	return dev;
}


int analyzer_set_observ( int dev, struct observ *obs)
{
	int i;
	int sweep_on;
	char status;
	double v;
	char buff[(AN_MAX_POINTS << 3) + 2];
	char *np = buff;
        char *endp;

	/* TODO: setear las llaves (falta informacion)
	   ALGUNA_FUNCION(obs->mode);
	*/

	/* TODO: Que hago con las repeticiones: sumo???, Martin ?? */
	
	MY_IBWRT(dev,"FA %8.6fGHZ;", obs->start_freq);
	MY_IBWRT(dev,"FB %8.6fGHZ;", obs->stop_freq);
	MY_IBWRT(dev,"RB %dKHZ;", obs->RBW);
	MY_IBWRT(dev,"ST %dSC;", obs->sweep);
	
	MY_IBWRT(dev, "RQS %d;", RQS_EOS);  /* End Of Sweep */
	MY_IBWRT(dev, "TDF P;");
	MY_IBWRT(dev, "TS;");

	if ( my_ibwrta(dev,"TRA?;") & ERR )
	  return -1;

	sweep_on = 1;
	do{
		ibrsp(dev, &status);
		fprintf(stderr,"# ibrsp->status: 0x%x\n", status);
		if(ibsta & ERR){
			/* TODO: 
			   Reestableser sistema
			*/
			return -1;			
		}
		if(status & LACS){
			sweep_on = 0;
		}else{
			sleep(1);
		}
	}while(sweep_on);

	fprintf(stderr,"# Reading.\n");
	ibrd(dev, buff, (AN_MAX_POINTS << 3) + 2);
	/* bytes_read = ibcnt; */
	//printf("[%s]\n", buff);

	for(i=0;i<401;i++){
	  v =  strtof(np, &endp);
	  np = endp + 1;
	  printf("%6.2f\n", v);

        }
	

	return 0;
}
